@@ -35,8 +35,6 @@
 
 
 
-static void         thunar_abstract_icon_view_class_init            (ThunarAbstractIconViewClass  *klass);
-static void         thunar_abstract_icon_view_init                  (ThunarAbstractIconView       *abstract_icon_view);
 static void         thunar_abstract_icon_view_style_set             (GtkWidget                    *widget,
                                                                      GtkStyle                     *previous_style);
 static void         thunar_abstract_icon_view_connect_ui_manager    (ThunarStandardView           *standard_view,
@@ -133,36 +131,7 @@ static const GtkRadioActionEntry order_action_entries[] =
 
 
 
-static GObjectClass *thunar_abstract_icon_view_parent_class;
-
-
-
-GType
-thunar_abstract_icon_view_get_type (void)
-{
-  static GType type = G_TYPE_INVALID;
-
-  if (G_UNLIKELY (type == G_TYPE_INVALID))
-    {
-      static const GTypeInfo info =
-      {
-        sizeof (ThunarAbstractIconViewClass),
-        NULL,
-        NULL,
-        (GClassInitFunc) thunar_abstract_icon_view_class_init,
-        NULL,
-        NULL,
-        sizeof (ThunarAbstractIconView),
-        0,
-        (GInstanceInitFunc) thunar_abstract_icon_view_init,
-        NULL,
-      };
-
-      type = g_type_register_static (THUNAR_TYPE_STANDARD_VIEW, I_("ThunarAbstractIconView"), &info, G_TYPE_FLAG_ABSTRACT);
-    }
-
-  return type;
-}
+G_DEFINE_ABSTRACT_TYPE (ThunarAbstractIconView, thunar_abstract_icon_view, THUNAR_TYPE_STANDARD_VIEW)
 
 
 
@@ -175,9 +144,6 @@ thunar_abstract_icon_view_class_init (ThunarAbstractIconViewClass *klass)
   /* add private data to the instance type */
   g_type_class_add_private (klass, sizeof (ThunarAbstractIconViewPrivate));
 
-  /* determine the parent type class */
-  thunar_abstract_icon_view_parent_class = g_type_class_peek_parent (klass);
-
   gtkwidget_class = GTK_WIDGET_CLASS (klass);
   gtkwidget_class->style_set = thunar_abstract_icon_view_style_set;
 
@@ -642,10 +608,8 @@ thunar_abstract_icon_view_expose_event (ExoIconView            *view,
   GtkAction  *action = NULL;
   GdkPixbuf  *stock_icon = NULL;
   gchar      *stock_id;
-#if GTK_CHECK_VERSION(2,7,1)
   GdkColor    bg;
   cairo_t    *cr;
-#endif
 
   _thunar_return_val_if_fail (EXO_IS_ICON_VIEW (view), FALSE);
   _thunar_return_val_if_fail (THUNAR_IS_ABSTRACT_ICON_VIEW (abstract_icon_view), FALSE);
@@ -654,7 +618,6 @@ thunar_abstract_icon_view_expose_event (ExoIconView            *view,
   _thunar_return_val_if_fail (abstract_icon_view->priv->gesture_release_id > 0, FALSE);
 
   /* shade the abstract_icon view content while performing mouse gestures */
-#if GTK_CHECK_VERSION(2,7,1)
   cr = gdk_cairo_create (event->window);
   bg = GTK_WIDGET (view)->style->base[GTK_STATE_NORMAL];
   cairo_set_source_rgba (cr, bg.red / 65535.0, bg.green / 65535.0, bg.blue / 65535.0, 0.7);
@@ -662,7 +625,6 @@ thunar_abstract_icon_view_expose_event (ExoIconView            *view,
   cairo_clip (cr);
   cairo_paint (cr);
   cairo_destroy (cr);
-#endif
 
   /* determine the gesture action */
   action = thunar_abstract_icon_view_gesture_action (abstract_icon_view);
@@ -792,11 +754,11 @@ thunar_abstract_icon_view_sort_column_changed (GtkTreeSortable        *sortable,
     {
       /* apply the new sort column */
       action = gtk_action_group_get_action (THUNAR_STANDARD_VIEW (abstract_icon_view)->action_group, "sort-by-name");
-      exo_gtk_radio_action_set_current_value (GTK_RADIO_ACTION (action), column);
+      gtk_radio_action_set_current_value (GTK_RADIO_ACTION (action), column);
 
       /* apply the new sort order */
       action = gtk_action_group_get_action (THUNAR_STANDARD_VIEW (abstract_icon_view)->action_group, "sort-ascending");
-      exo_gtk_radio_action_set_current_value (GTK_RADIO_ACTION (action), order);
+      gtk_radio_action_set_current_value (GTK_RADIO_ACTION (action), order);
     }
 }
 
