@@ -1,6 +1,7 @@
 /* $Id$ */
 /*-
  * Copyright (c) 2006-2007 Benedikt Meurer <benny@xfce.org>
+ * Copyright (c) 2009 Jannis Pohlmann <jannis@xfce.org>
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the Free
@@ -41,7 +42,7 @@ typedef enum
   THUNAR_RENAMER_MODE_BOTH,
 } ThunarRenamerMode;
 
-GType thunar_renamer_mode_get_type (void) G_GNUC_CONST G_GNUC_INTERNAL;
+GType thunar_renamer_mode_get_type (void) G_GNUC_CONST;
 
 
 #define THUNAR_TYPE_COLOR_STYLE (thunar_color_style_get_type ())
@@ -61,7 +62,7 @@ typedef enum
   THUNAR_COLOR_STYLE_VGRADIENT,
 } ThunarColorStyle;
 
-GType thunar_color_style_get_type (void) G_GNUC_CONST G_GNUC_INTERNAL;
+GType thunar_color_style_get_type (void) G_GNUC_CONST;
 
 
 #define THUNAR_TYPE_DATE_STYLE (thunar_date_style_get_type ())
@@ -83,7 +84,7 @@ typedef enum
   THUNAR_DATE_STYLE_ISO,
 } ThunarDateStyle;
 
-GType thunar_date_style_get_type (void) G_GNUC_CONST G_GNUC_INTERNAL;
+GType thunar_date_style_get_type (void) G_GNUC_CONST;
 
 
 #define THUNAR_TYPE_COLUMN (thunar_column_get_type ())
@@ -129,7 +130,7 @@ typedef enum
   THUNAR_N_VISIBLE_COLUMNS = THUNAR_COLUMN_FILE,
 } ThunarColumn;
 
-GType thunar_column_get_type (void) G_GNUC_CONST G_GNUC_INTERNAL;
+GType thunar_column_get_type (void) G_GNUC_CONST;
 
 
 #define THUNAR_TYPE_ICON_SIZE (thunar_icon_size_get_type ())
@@ -150,14 +151,14 @@ typedef enum
 {
   THUNAR_ICON_SIZE_SMALLEST = 16,
   THUNAR_ICON_SIZE_SMALLER  = 24,
-  THUNAR_ICON_SIZE_SMALL    = 36,
+  THUNAR_ICON_SIZE_SMALL    = 32,
   THUNAR_ICON_SIZE_NORMAL   = 48,
   THUNAR_ICON_SIZE_LARGE    = 64,
   THUNAR_ICON_SIZE_LARGER   = 96,
   THUNAR_ICON_SIZE_LARGEST  = 128,
 } ThunarIconSize;
 
-GType thunar_icon_size_get_type (void) G_GNUC_CONST G_GNUC_INTERNAL;
+GType thunar_icon_size_get_type (void) G_GNUC_CONST;
 
 
 #define THUNAR_TYPE_RECURSIVE_PERMISSIONS (thunar_recursive_permissions_get_type ())
@@ -177,7 +178,7 @@ typedef enum
   THUNAR_RECURSIVE_PERMISSIONS_NEVER,
 } ThunarRecursivePermissionsMode;
 
-GType thunar_recursive_permissions_get_type (void) G_GNUC_CONST G_GNUC_INTERNAL;
+GType thunar_recursive_permissions_get_type (void) G_GNUC_CONST;
 
 
 #define THUNAR_TYPE_WALLPAPER_STYLE (thunar_wallpaper_style_get_type ())
@@ -199,7 +200,7 @@ typedef enum
   THUNAR_WALLPAPER_STYLE_TILED,
 } ThunarWallpaperStyle;
 
-GType thunar_wallpaper_style_get_type (void) G_GNUC_CONST G_GNUC_INTERNAL;
+GType thunar_wallpaper_style_get_type (void) G_GNUC_CONST;
 
 
 #define THUNAR_TYPE_ZOOM_LEVEL (thunar_zoom_level_get_type ())
@@ -231,8 +232,63 @@ typedef enum
   THUNAR_ZOOM_N_LEVELS,
 } ThunarZoomLevel;
 
-GType          thunar_zoom_level_get_type     (void) G_GNUC_CONST G_GNUC_INTERNAL;
-ThunarIconSize thunar_zoom_level_to_icon_size (ThunarZoomLevel zoom_level) G_GNUC_CONST G_GNUC_INTERNAL;
+GType          thunar_zoom_level_get_type     (void) G_GNUC_CONST;
+ThunarIconSize thunar_zoom_level_to_icon_size (ThunarZoomLevel zoom_level) G_GNUC_CONST;
+
+
+#define THUNAR_TYPE_JOB_RESPONSE (thunar_job_response_get_type ())
+
+/**
+ * ThunarJobResponse:
+ * @THUNAR_JOB_RESPONSE_YES     :
+ * @THUNAR_JOB_RESPONSE_YES_ALL :
+ * @THUNAR_JOB_RESPONSE_NO      :
+ * @THUNAR_JOB_RESPONSE_NO_ALL  :
+ * @THUNAR_JOB_RESPONSE_CANCEL  :
+ * @THUNAR_JOB_RESPONSE_RETRY   :
+ *
+ * Possible responses for the ThunarJob::ask signal.
+ **/
+typedef enum /*< flags >*/
+{
+  THUNAR_JOB_RESPONSE_YES     = 1 << 0,
+  THUNAR_JOB_RESPONSE_YES_ALL = 1 << 1,
+  THUNAR_JOB_RESPONSE_NO      = 1 << 2,
+  THUNAR_JOB_RESPONSE_CANCEL  = 1 << 3,
+  THUNAR_JOB_RESPONSE_NO_ALL  = 1 << 4,
+  THUNAR_JOB_RESPONSE_RETRY   = 1 << 5,
+} ThunarJobResponse;
+
+GType thunar_job_response_get_type (void) G_GNUC_CONST;
+
+
+#define THUNAR_TYPE_FILE_MODE (thunar_file_mode_get_type ())
+
+/**
+ * ThunarFileMode:
+ *
+ * Special flags and permissions of a filesystem entity.
+ **/
+typedef enum /*< flags >*/
+{
+  THUNAR_FILE_MODE_SUID       = 04000,
+  THUNAR_FILE_MODE_SGID       = 02000,
+  THUNAR_FILE_MODE_STICKY     = 01000,
+  THUNAR_FILE_MODE_USR_ALL    = 00700,
+  THUNAR_FILE_MODE_USR_READ   = 00400,
+  THUNAR_FILE_MODE_USR_WRITE  = 00200,
+  THUNAR_FILE_MODE_USR_EXEC   = 00100,
+  THUNAR_FILE_MODE_GRP_ALL    = 00070,
+  THUNAR_FILE_MODE_GRP_READ   = 00040,
+  THUNAR_FILE_MODE_GRP_WRITE  = 00020,
+  THUNAR_FILE_MODE_GRP_EXEC   = 00010,
+  THUNAR_FILE_MODE_OTH_ALL    = 00007,
+  THUNAR_FILE_MODE_OTH_READ   = 00004,
+  THUNAR_FILE_MODE_OTH_WRITE  = 00002,
+  THUNAR_FILE_MODE_OTH_EXEC   = 00001,
+} ThunarFileMode;
+
+GType thunar_file_mode_get_type (void) G_GNUC_CONST;
 
 G_END_DECLS;
 
