@@ -41,8 +41,6 @@ enum
 
 
 
-static void thunar_icon_renderer_class_init    (ThunarIconRendererClass *klass);
-static void thunar_icon_renderer_init          (ThunarIconRenderer      *icon_renderer);
 static void thunar_icon_renderer_finalize      (GObject                 *object);
 static void thunar_icon_renderer_get_property  (GObject                 *object,
                                                 guint                    prop_id,
@@ -69,36 +67,7 @@ static void thunar_icon_renderer_render        (GtkCellRenderer         *rendere
 
 
 
-static GObjectClass *thunar_icon_renderer_parent_class;
-
-
-
-GType
-thunar_icon_renderer_get_type (void)
-{
-  static GType type = G_TYPE_INVALID;
-
-  if (G_UNLIKELY (type == G_TYPE_INVALID))
-    {
-      static const GTypeInfo info =
-      {
-        sizeof (ThunarIconRendererClass),
-        NULL,
-        NULL,
-        (GClassInitFunc) thunar_icon_renderer_class_init,
-        NULL,
-        NULL,
-        sizeof (ThunarIconRenderer),
-        0,
-        (GInstanceInitFunc) thunar_icon_renderer_init,
-        NULL,
-      };
-
-      type = g_type_register_static (GTK_TYPE_CELL_RENDERER, I_("ThunarIconRenderer"), &info, 0);
-    }
-
-  return type;
-}
+G_DEFINE_TYPE (ThunarIconRenderer, thunar_icon_renderer, GTK_TYPE_CELL_RENDERER)
 
 
 
@@ -108,9 +77,6 @@ thunar_icon_renderer_class_init (ThunarIconRendererClass *klass)
   GtkCellRendererClass *gtkcell_renderer_class;
   GObjectClass         *gobject_class;
 
-  /* determine the parent type class */
-  thunar_icon_renderer_parent_class = g_type_class_peek_parent (klass);
-
   gobject_class = G_OBJECT_CLASS (klass);
   gobject_class->finalize = thunar_icon_renderer_finalize;
   gobject_class->get_property = thunar_icon_renderer_get_property;
@@ -504,10 +470,10 @@ thunar_icon_renderer_render (GtkCellRenderer     *renderer,
               emblem_area.height = gdk_pixbuf_get_height (emblem);
 
               /* shrink insane emblems */
-              if (G_UNLIKELY (MAX (emblem_area.width, emblem_area.height) > (gint) MIN ((2 * icon_renderer->size) / 3, 36)))
+              if (G_UNLIKELY (MAX (emblem_area.width, emblem_area.height) > (gint) MIN ((2 * icon_renderer->size) / 3, 32)))
                 {
                   /* scale down the emblem */
-                  temp = exo_gdk_pixbuf_scale_ratio (emblem, MIN ((2 * icon_renderer->size) / 3, 36));
+                  temp = exo_gdk_pixbuf_scale_ratio (emblem, MIN ((2 * icon_renderer->size) / 3, 32));
                   g_object_unref (G_OBJECT (emblem));
                   emblem = temp;
 
