@@ -1,6 +1,7 @@
-/* $Id: thunar-list-model.c 28811 2008-11-15 12:06:38Z jannis $ */
+/* $Id$ */
 /*-
  * Copyright (c) 2004-2007 Benedikt Meurer <benny@xfce.org>
+ * Copyright (c) 2009-2011 Jannis Pohlmann <jannis@xfce.org>
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the Free
@@ -28,10 +29,12 @@
 #include <string.h>
 #endif
 
+#include <thunar/thunar-application.h>
 #include <thunar/thunar-file-monitor.h>
 #include <thunar/thunar-gobject-extensions.h>
 #include <thunar/thunar-list-model.h>
 #include <thunar/thunar-private.h>
+#include <thunar/thunar-user.h>
 
 
 
@@ -65,11 +68,9 @@ typedef struct _SortTuple SortTuple;
 
 
 
-static void               thunar_list_model_class_init            (ThunarListModelClass   *klass);
 static void               thunar_list_model_tree_model_init       (GtkTreeModelIface      *iface);
 static void               thunar_list_model_drag_dest_init        (GtkTreeDragDestIface   *iface);
 static void               thunar_list_model_sortable_init         (GtkTreeSortableIface   *iface);
-static void               thunar_list_model_init                  (ThunarListModel        *store);
 static void               thunar_list_model_finalize              (GObject                *object);
 static void               thunar_list_model_get_property          (GObject                *object,
                                                                    guint                   prop_id,
@@ -82,7 +83,7 @@ static void               thunar_list_model_set_property          (GObject
 static GtkTreeModelFlags  thunar_list_model_get_flags             (GtkTreeModel           *model);
 static gint               thunar_list_model_get_n_columns         (GtkTreeModel           *model);
 static GType              thunar_list_model_get_column_type       (GtkTreeModel           *model,
-                                                                   gint                    index);
+                                                                   gint                    idx);
 static gboolean           thunar_list_model_get_iter              (GtkTreeModel           *model,
                                                                    GtkTreeIter            *iter,
                                                                    GtkTreePath            *path);
@@ -123,12 +124,12 @@ static void               thunar_list_model_set_sort_column_id    (GtkTreeSortab
 static void               thunar_list_model_set_default_sort_func (GtkTreeSortable        *sortable,
                                                                    GtkTreeIterCompareFunc  func,
                                                                    gpointer                data,
-                                                                   GtkDestroyNotify        destroy);
+                                                                   GDestroyNotify          destroy);
 static void               thunar_list_model_set_sort_func         (GtkTreeSortable        *sortable,
                                                                    gint                    sort_column_id,
                                                                    GtkTreeIterCompareFunc  func,
                                                                    gpointer                data,
-                                                                   GtkDestroyNotify        destroy);
+                                                                   GDestroyNotify          destroy);
 static gboolean           thunar_list_model_has_default_sort_func (GtkTreeSortable        *sortable);
 static gint               thunar_list_model_cmp                   (ThunarListModel        *store,
                                                                    ThunarFile             *a,
@@ -224,8 +225,6 @@ struct _ThunarListModel
    */
   ThunarFileMonitor      *file_monitor;
 
-  ThunarVfsVolumeManager *volume_manager;
-
   /* ids for the "row-inserted" and "row-deleted" signals
    * of GtkTreeModel to speed up folder changing.
    */
@@ -249,61 +248,14 @@ struct _SortTuple
 
 
 
-static GObjectClass *thunar_list_model_parent_class;
-static guint         list_model_signals[LAST_SIGNAL];
+static guint list_model_signals[LAST_SIGNAL];
 
 
 
-GType
-thunar_list_model_get_type (void)
-{
-  static GType type = G_TYPE_INVALID;
-
-  if (G_UNLIKELY (type == G_TYPE_INVALID))
-    {
-      static const GTypeInfo info =
-      {
-        sizeof (ThunarListModelClass),
-        NULL,
-        NULL,
-        (GClassInitFunc) thunar_list_model_class_init,
-        NULL,
-        NULL,
-        sizeof (ThunarListModel),
-        0,
-        (GInstanceInitFunc) thunar_list_model_init,
-        NULL,
-      };
-
-      static const GInterfaceInfo tree_model_info =
-      {
-        (GInterfaceInitFunc) thunar_list_model_tree_model_init,
-        NULL,
-        NULL,
-      };
-
-      static const GInterfaceInfo drag_dest_info =
-      {
-        (GInterfaceInitFunc) thunar_list_model_drag_dest_init,
-        NULL,
-        NULL,
-      };
-
-      static const GInterfaceInfo sortable_info =
-      {
-        (GInterfaceInitFunc) thunar_list_model_sortable_init,
-        NULL,
-        NULL,
-      };
-
-      type = g_type_register_static (G_TYPE_OBJECT, I_("ThunarListModel"), &info, 0);
-      g_type_add_interface_static (type, GTK_TYPE_TREE_MODEL, &tree_model_info);
-      g_type_add_interface_static (type, GTK_TYPE_TREE_DRAG_DEST, &drag_dest_info);
-      g_type_add_interface_static (type, GTK_TYPE_TREE_SORTABLE, &sortable_info);
-    }
-
-  return type;
-}
+G_DEFINE_TYPE_WITH_CODE (ThunarListModel, thunar_list_model, G_TYPE_OBJECT,
+    G_IMPLEMENT_INTERFACE (GTK_TYPE_TREE_MODEL, thunar_list_model_tree_model_init)
+    G_IMPLEMENT_INTERFACE (GTK_TYPE_TREE_DRAG_DEST, thunar_list_model_drag_dest_init)
+    G_IMPLEMENT_INTERFACE (GTK_TYPE_TREE_SORTABLE, thunar_list_model_sortable_init))
 
 
 
@@ -312,9 +264,6 @@ thunar_list_model_class_init (ThunarListModelClass *klass)
 {
   GObjectClass *gobject_class;
 
-  /* determine the parent type class */
-  thunar_list_model_parent_class = g_type_class_peek_parent (klass);
-
   gobject_class               = G_OBJECT_CLASS (klass);
   gobject_class->finalize     = thunar_list_model_finalize;
   gobject_class->get_property = thunar_list_model_get_property;
@@ -467,8 +416,6 @@ thunar_list_model_init (ThunarListModel *store)
   store->stamp                = g_random_int ();
 #endif
 
-  store->volume_manager       = thunar_vfs_volume_manager_get_default ();
-
   store->row_inserted_id      = g_signal_lookup ("row-inserted", GTK_TYPE_TREE_MODEL);
   store->row_deleted_id       = g_signal_lookup ("row-deleted", GTK_TYPE_TREE_MODEL);
 
@@ -481,7 +428,8 @@ thunar_list_model_init (ThunarListModel *store)
    * connect "changed" to every single ThunarFile we own.
    */
   store->file_monitor = thunar_file_monitor_get_default ();
-  g_signal_connect (G_OBJECT (store->file_monitor), "file-changed", G_CALLBACK (thunar_list_model_file_changed), store);
+  g_signal_connect (G_OBJECT (store->file_monitor), "file-changed", 
+                    G_CALLBACK (thunar_list_model_file_changed), store);
 }
 
 
@@ -494,9 +442,6 @@ thunar_list_model_finalize (GObject *object)
   /* unlink from the folder (if any) */
   thunar_list_model_set_folder (store, NULL);
 
-  /* disconnect from the volume manager */
-  g_object_unref (G_OBJECT (store->volume_manager));
-
   /* disconnect from the file monitor */
   g_signal_handlers_disconnect_by_func (G_OBJECT (store->file_monitor), thunar_list_model_file_changed, store);
   g_object_unref (G_OBJECT (store->file_monitor));
@@ -604,9 +549,9 @@ thunar_list_model_get_n_columns (GtkTreeModel *model)
 
 static GType
 thunar_list_model_get_column_type (GtkTreeModel *model,
-                                   gint          index)
+                                   gint          idx)
 {
-  switch (index)
+  switch (idx)
     {
     case THUNAR_COLUMN_DATE_ACCESSED:
       return G_TYPE_STRING;
@@ -677,14 +622,14 @@ thunar_list_model_get_path (GtkTreeModel *model,
                             GtkTreeIter  *iter)
 {
   ThunarListModel *store = THUNAR_LIST_MODEL (model);
-  gint             index = 0;
+  gint             idx = 0;
 
   _thunar_return_val_if_fail (THUNAR_IS_LIST_MODEL (store), NULL);
   _thunar_return_val_if_fail (iter->stamp == store->stamp, NULL);
 
-  index = g_slist_position (store->rows, iter->user_data);
-  if (G_LIKELY (index >= 0))
-    return gtk_tree_path_new_from_indices (index, -1);
+  idx = g_slist_position (store->rows, iter->user_data);
+  if (G_LIKELY (idx >= 0))
+    return gtk_tree_path_new_from_indices (idx, -1);
 
   return NULL;
 }
@@ -697,13 +642,13 @@ thunar_list_model_get_value (GtkTreeModel *model,
                              gint          column,
                              GValue       *value)
 {
-  ThunarVfsMimeInfo *mime_info;
-  ThunarVfsGroup    *group;
-  ThunarVfsUser     *user;
-  const gchar       *name;
-  const gchar       *real_name;
-  ThunarFile        *file;
-  gchar             *str;
+  ThunarGroup *group;
+  const gchar *content_type;
+  const gchar *name;
+  const gchar *real_name;
+  ThunarUser  *user;
+  ThunarFile  *file;
+  gchar       *str;
 
   _thunar_return_if_fail (THUNAR_IS_LIST_MODEL (model));
   _thunar_return_if_fail (iter->stamp == (THUNAR_LIST_MODEL (model))->stamp);
@@ -729,7 +674,7 @@ thunar_list_model_get_value (GtkTreeModel *model,
       group = thunar_file_get_group (file);
       if (G_LIKELY (group != NULL))
         {
-          g_value_set_string (value, thunar_vfs_group_get_name (group));
+          g_value_set_string (value, thunar_group_get_name (group));
           g_object_unref (G_OBJECT (group));
         }
       else
@@ -740,8 +685,7 @@ thunar_list_model_get_value (GtkTreeModel *model,
 
     case THUNAR_COLUMN_MIME_TYPE:
       g_value_init (value, G_TYPE_STRING);
-      mime_info = thunar_file_get_mime_info (file);
-      g_value_set_static_string (value, thunar_vfs_mime_info_get_name (mime_info));
+      g_value_set_static_string (value, thunar_file_get_content_type (file));
       break;
 
     case THUNAR_COLUMN_NAME:
@@ -755,8 +699,8 @@ thunar_list_model_get_value (GtkTreeModel *model,
       if (G_LIKELY (user != NULL))
         {
           /* determine sane display name for the owner */
-          name = thunar_vfs_user_get_name (user);
-          real_name = thunar_vfs_user_get_real_name (user);
+          name = thunar_user_get_name (user);
+          real_name = thunar_user_get_real_name (user);
           str = G_LIKELY (real_name != NULL) ? g_strdup_printf ("%s (%s)", real_name, name) : g_strdup (name);
           g_value_take_string (value, str);
           g_object_unref (G_OBJECT (user));
@@ -779,13 +723,14 @@ thunar_list_model_get_value (GtkTreeModel *model,
 
     case THUNAR_COLUMN_TYPE:
       g_value_init (value, G_TYPE_STRING);
-      mime_info = thunar_file_get_mime_info (file);
-      if (G_UNLIKELY (strcmp (thunar_vfs_mime_info_get_name (mime_info), "inode/symlink") == 0))
-        g_value_set_static_string (value, _("broken link"));
-      else if (G_UNLIKELY (thunar_file_is_symlink (file)))
-        g_value_take_string (value, g_strdup_printf (_("link to %s"), thunar_vfs_mime_info_get_comment (mime_info)));
+      if (G_UNLIKELY (thunar_file_is_symlink (file)))
+        g_value_take_string (value, g_strdup_printf (_("link to %s"), thunar_file_get_symlink_target (file)));
       else
-        g_value_set_static_string (value, thunar_vfs_mime_info_get_comment (mime_info));
+        {
+          content_type = thunar_file_get_content_type (file);
+          if (content_type != NULL)
+            g_value_take_string (value, g_content_type_get_description (content_type));
+        }
       break;
 
     case THUNAR_COLUMN_FILE:
@@ -795,7 +740,7 @@ thunar_list_model_get_value (GtkTreeModel *model,
 
     case THUNAR_COLUMN_FILE_NAME:
       g_value_init (value, G_TYPE_STRING);
-      g_value_take_string (value, g_filename_display_name (thunar_vfs_path_get_name (thunar_file_get_path (file))));
+      g_value_set_static_string (value, thunar_file_get_basename (file));
       break;
 
     default:
@@ -1027,9 +972,9 @@ static void
 thunar_list_model_set_default_sort_func (GtkTreeSortable       *sortable,
                                          GtkTreeIterCompareFunc func,
                                          gpointer               data,
-                                         GtkDestroyNotify       destroy)
+                                         GDestroyNotify         destroy)
 {
-  g_error ("ThunarListModel has sorting facilities built-in!");
+  g_critical ("ThunarListModel has sorting facilities built-in!");
 }
 
 
@@ -1039,9 +984,9 @@ thunar_list_model_set_sort_func (GtkTreeSortable       *sortable,
                                  gint                   sort_column_id,
                                  GtkTreeIterCompareFunc func,
                                  gpointer               data,
-                                 GtkDestroyNotify       destroy)
+                                 GDestroyNotify         destroy)
 {
-  g_error ("ThunarListModel has sorting facilities built-in!");
+  g_critical ("ThunarListModel has sorting facilities built-in!");
 }
 
 
@@ -1267,7 +1212,7 @@ thunar_list_model_folder_destroy (ThunarFolder    *folder,
 
   thunar_list_model_set_folder (store, NULL);
 
-  // TODO: What to do when the folder is deleted?
+  /* TODO: What to do when the folder is deleted? */
 }
 
 
@@ -1301,7 +1246,7 @@ thunar_list_model_files_added (ThunarFolder    *folder,
   GSList      *prev = NULL;
   GSList      *row;
   gint        *indices;
-  gint         index = 0;
+  gint         idx = 0;
 
   /* we use a simple trick here to avoid allocating
    * GtkTreePath's again and again, by simply accessing
@@ -1310,7 +1255,7 @@ thunar_list_model_files_added (ThunarFolder    *folder,
    */
   path = gtk_tree_path_new_from_indices (0, -1);
   indices = gtk_tree_path_get_indices (path);
-
+      
   /* process all added files */
   for (; files != NULL; files = files->next)
     {
@@ -1333,7 +1278,7 @@ thunar_list_model_files_added (ThunarFolder    *folder,
           if (G_UNLIKELY (store->rows == NULL || thunar_list_model_cmp (store, file, store->rows->data) < 0))
             {
               prev        = NULL;
-              index       = 0;
+              idx       = 0;
               row->next   = store->rows;
               store->rows = row;
             }
@@ -1346,10 +1291,10 @@ thunar_list_model_files_added (ThunarFolder    *folder,
               if (G_UNLIKELY (prev == NULL || thunar_list_model_cmp (store, file, prev->data) < 0))
                 {
                   prev = store->rows;
-                  index = 1;
+                  idx = 1;
                 }
 
-              for (; prev->next != NULL; ++index, prev = prev->next)
+              for (; prev->next != NULL; ++idx, prev = prev->next)
                 if (thunar_list_model_cmp (store, file, prev->next->data) < 0)
                   break;
 
@@ -1363,7 +1308,7 @@ thunar_list_model_files_added (ThunarFolder    *folder,
           GTK_TREE_ITER_INIT (iter, store->stamp, row);
 
           /* tell the view about the new item */
-          indices[0] = index;
+          indices[0] = idx;
           gtk_tree_model_row_inserted (GTK_TREE_MODEL (store), path, &iter);
         }
     }
@@ -1420,8 +1365,8 @@ sort_by_date_accessed (const ThunarFile *a,
                        const ThunarFile *b,
                        gboolean          case_sensitive)
 {
-  ThunarVfsFileTime date_a;
-  ThunarVfsFileTime date_b;
+  guint64 date_a;
+  guint64 date_b;
 
   date_a = thunar_file_get_date (a, THUNAR_FILE_DATE_ACCESSED);
   date_b = thunar_file_get_date (b, THUNAR_FILE_DATE_ACCESSED);
@@ -1441,8 +1386,8 @@ sort_by_date_modified (const ThunarFile *a,
                        const ThunarFile *b,
                        gboolean          case_sensitive)
 {
-  ThunarVfsFileTime date_a;
-  ThunarVfsFileTime date_b;
+  guint64 date_a;
+  guint64 date_b;
 
   date_a = thunar_file_get_date (a, THUNAR_FILE_DATE_MODIFIED);
   date_b = thunar_file_get_date (b, THUNAR_FILE_DATE_MODIFIED);
@@ -1462,10 +1407,10 @@ sort_by_file_name (const ThunarFile *a,
                    const ThunarFile *b,
                    gboolean          case_sensitive)
 {
-  const gchar *a_name = thunar_vfs_path_get_name (thunar_file_get_path (a));
-  const gchar *b_name = thunar_vfs_path_get_name (thunar_file_get_path (b));
+  const gchar *a_name = thunar_file_get_display_name (a);
+  const gchar *b_name = thunar_file_get_display_name (b);
 
-  if (G_UNLIKELY (!case_sensitive))
+  if (!case_sensitive)
     return strcasecmp (a_name, b_name);
   else
     return strcmp (a_name, b_name);
@@ -1478,12 +1423,51 @@ sort_by_group (const ThunarFile *a,
                const ThunarFile *b,
                gboolean          case_sensitive)
 {
-  if (a->info->gid < b->info->gid)
-    return -1;
-  else if (a->info->gid > b->info->gid)
-    return 1;
-  else
+  ThunarGroup *group_a;
+  ThunarGroup *group_b;
+  guint32      gid_a;
+  guint32      gid_b;
+  gint         result;
+
+  if (thunar_file_get_info (a) == NULL || thunar_file_get_info (b) == NULL)
     return sort_by_name (a, b, case_sensitive);
+
+  group_a = thunar_file_get_group (a);
+  group_b = thunar_file_get_group (b);
+
+  if (group_a != NULL && group_b != NULL)
+    {
+      if (!case_sensitive)
+        {
+          result = strcasecmp (thunar_group_get_name (group_a),
+                               thunar_group_get_name (group_b));
+        }
+      else
+        {
+          result = strcmp (thunar_group_get_name (group_a),
+                           thunar_group_get_name (group_b));
+        }
+    }
+  else
+    {
+      gid_a = g_file_info_get_attribute_uint32 (thunar_file_get_info (a),
+                                                G_FILE_ATTRIBUTE_UNIX_GID);
+      gid_b = g_file_info_get_attribute_uint32 (thunar_file_get_info (b),
+                                                G_FILE_ATTRIBUTE_UNIX_GID);
+      
+      result = CLAMP ((gint) gid_a - (gint) gid_b, -1, 1);
+    }
+
+  if (group_a != NULL)
+    g_object_unref (group_a);
+
+  if (group_b != NULL)
+    g_object_unref (group_b);
+  
+  if (result == 0)
+    return sort_by_name (a, b, case_sensitive);
+  else
+    return result;
 }
 
 
@@ -1493,15 +1477,14 @@ sort_by_mime_type (const ThunarFile *a,
                    const ThunarFile *b,
                    gboolean          case_sensitive)
 {
-  const ThunarVfsMimeInfo *info_a;
-  const ThunarVfsMimeInfo *info_b;
-  gint                     result;
+  const gchar *content_type_a;
+  const gchar *content_type_b;
+  gint         result;
 
-  info_a = thunar_file_get_mime_info (a);
-  info_b = thunar_file_get_mime_info (b);
+  content_type_a = thunar_file_get_content_type (a);
+  content_type_b = thunar_file_get_content_type (b);
 
-  result = strcasecmp (thunar_vfs_mime_info_get_name (info_a),
-                       thunar_vfs_mime_info_get_name (info_b));
+  result = strcasecmp (content_type_a, content_type_b);
 
   if (result == 0)
     result = sort_by_name (a, b, case_sensitive);
@@ -1526,12 +1509,58 @@ sort_by_owner (const ThunarFile *a,
                const ThunarFile *b,
                gboolean          case_sensitive)
 {
-  if (a->info->uid < b->info->uid)
-    return -1;
-  else if (a->info->uid > b->info->uid)
-    return 1;
-  else
+  const gchar *name_a;
+  const gchar *name_b;
+  const gchar *real_name_a;
+  const gchar *real_name_b;
+  ThunarUser  *user_a;
+  ThunarUser  *user_b;
+  guint32      uid_a;
+  guint32      uid_b;
+  gchar       *str_a;
+  gchar       *str_b;
+  gint         result;
+
+  if (thunar_file_get_info (a) == NULL || thunar_file_get_info (b) == NULL)
     return sort_by_name (a, b, case_sensitive);
+
+  user_a = thunar_file_get_user (a);
+  user_b = thunar_file_get_user (b);
+
+  if (user_a != NULL && user_b != NULL)
+    {
+      name_a = thunar_user_get_name (user_a);
+      real_name_a = thunar_user_get_real_name (user_a);
+      str_a = G_LIKELY (real_name_a != NULL) 
+        ? g_strdup_printf ("%s (%s)", real_name_a, name_a) : g_strdup (name_a);
+
+      name_b = thunar_user_get_name (user_b);
+      real_name_b = thunar_user_get_real_name (user_b);
+      str_b = G_LIKELY (real_name_b != NULL) 
+        ? g_strdup_printf ("%s (%s)", real_name_b, name_b) : g_strdup (name_b);
+
+      if (!case_sensitive)
+        result = strcasecmp (str_a, str_b);
+      else
+        result = strcmp (str_a, str_b);
+
+      g_free (str_a);
+      g_free (str_b);
+    }
+  else
+    {
+      uid_a = g_file_info_get_attribute_uint32 (thunar_file_get_info (a),
+                                                G_FILE_ATTRIBUTE_UNIX_UID);
+      uid_b = g_file_info_get_attribute_uint32 (thunar_file_get_info (b),
+                                                G_FILE_ATTRIBUTE_UNIX_UID);
+
+      result = CLAMP ((gint) uid_a - (gint) uid_b, -1, 1);
+    }
+
+  if (result == 0)
+    return sort_by_name (a, b, case_sensitive);
+  else
+    return result;
 }
 
 
@@ -1541,8 +1570,8 @@ sort_by_permissions (const ThunarFile *a,
                      const ThunarFile *b,
                      gboolean          case_sensitive)
 {
-  ThunarVfsFileMode mode_a;
-  ThunarVfsFileMode mode_b;
+  ThunarFileMode mode_a;
+  ThunarFileMode mode_b;
 
   mode_a = thunar_file_get_mode (a);
   mode_b = thunar_file_get_mode (b);
@@ -1562,8 +1591,8 @@ sort_by_size (const ThunarFile *a,
               const ThunarFile *b,
               gboolean          case_sensitive)
 {
-  ThunarVfsFileSize size_a;
-  ThunarVfsFileSize size_b;
+  guint64 size_a;
+  guint64 size_b;
 
   size_a = thunar_file_get_size (a);
   size_b = thunar_file_get_size (b);
@@ -1583,20 +1612,58 @@ sort_by_type (const ThunarFile *a,
               const ThunarFile *b,
               gboolean          case_sensitive)
 {
-  ThunarVfsMimeInfo *info_a;
-  ThunarVfsMimeInfo *info_b;
-  gint               result;
+  const gchar *content_type_a;
+  const gchar *content_type_b;
+  gchar       *description_a = NULL;
+  gchar       *description_b = NULL;
+  gint         result;
 
-  info_a = thunar_file_get_mime_info (a);
-  info_b = thunar_file_get_mime_info (b);
+  /* we alter the description of symlinks here because they are 
+   * displayed as "link to ..." in the detailed list view as well */
 
-  result = strcasecmp (thunar_vfs_mime_info_get_comment (info_a),
-                       thunar_vfs_mime_info_get_comment (info_b));
+  if (thunar_file_is_symlink (a))
+    {
+      description_a = g_strdup_printf (_("link to %s"), 
+                                       thunar_file_get_symlink_target (a));
+    }
+  else
+    {
+      content_type_a = thunar_file_get_content_type (a);
+      description_a = g_content_type_get_description (content_type_a);
+    }
+  
+  if (thunar_file_is_symlink (b))
+    {
+      description_b = g_strdup_printf (_("link to %s"),
+                                       thunar_file_get_symlink_target (b));
+    }
+  else
+    {
+      content_type_b = thunar_file_get_content_type (b);
+      description_b = g_content_type_get_description (content_type_b);
+    }
+
+  /* avoid calling strcasecmp with NULL parameters */
+  if (description_a == NULL || description_b == NULL)
+    {
+      g_free (description_a);
+      g_free (description_b);
+
+      return 0;
+    }
+
+  if (!case_sensitive)
+    result = strcasecmp (description_a, description_b);
+  else
+    result = strcmp (description_a, description_b);
+
+  g_free (description_a);
+  g_free (description_b);
 
   if (result == 0)
-    result = sort_by_name (a, b, case_sensitive);
-
-  return result;
+    return sort_by_name (a, b, case_sensitive);
+  else
+    return result;
 }
 
 
@@ -1630,11 +1697,7 @@ ThunarListModel*
 thunar_list_model_new_with_folder (ThunarFolder *folder)
 {
   _thunar_return_val_if_fail (THUNAR_IS_FOLDER (folder), NULL);
-
-  /* allocate the new list model */
-  return g_object_new (THUNAR_TYPE_LIST_MODEL,
-                       "folder", folder,
-                       NULL);
+  return g_object_new (THUNAR_TYPE_LIST_MODEL, "folder", folder, NULL);
 }
 
 
@@ -2119,14 +2182,14 @@ thunar_list_model_get_paths_for_files (ThunarListModel *store,
 {
   GSList *row;
   GList  *paths = NULL;
-  gint    index = 0;
+  gint    idx = 0;
 
   _thunar_return_val_if_fail (THUNAR_IS_LIST_MODEL (store), NULL);
 
   /* find the rows for the given files */
-  for (row = store->rows; row != NULL; ++index, row = row->next)
+  for (row = store->rows; row != NULL; ++idx, row = row->next)
     if (g_list_find (files, row->data) != NULL)
-      paths = g_list_prepend (paths, gtk_tree_path_new_from_indices (index, -1));
+      paths = g_list_prepend (paths, gtk_tree_path_new_from_indices (idx, -1));
 
   return paths;
 }
@@ -2156,7 +2219,7 @@ thunar_list_model_get_paths_for_pattern (ThunarListModel *store,
   GPatternSpec *pspec;
   GSList       *row;
   GList        *paths = NULL;
-  gint          index = 0;
+  gint          idx = 0;
 
   _thunar_return_val_if_fail (THUNAR_IS_LIST_MODEL (store), NULL);
   _thunar_return_val_if_fail (g_utf8_validate (pattern, -1, NULL), NULL);
@@ -2165,9 +2228,9 @@ thunar_list_model_get_paths_for_pattern (ThunarListModel *store,
   pspec = g_pattern_spec_new (pattern);
 
   /* find all rows that match the given pattern */
-  for (row = store->rows; row != NULL; ++index, row = row->next)
+  for (row = store->rows; row != NULL; ++idx, row = row->next)
     if (g_pattern_match_string (pspec, thunar_file_get_display_name (row->data)))
-      paths = g_list_prepend (paths, gtk_tree_path_new_from_indices (index, -1));
+      paths = g_list_prepend (paths, gtk_tree_path_new_from_indices (idx, -1));
 
   /* release the pattern */
   g_pattern_spec_free (pspec);
@@ -2200,22 +2263,23 @@ gchar*
 thunar_list_model_get_statusbar_text (ThunarListModel *store,
                                       GList           *selected_items)
 {
-  ThunarVfsMimeInfo *mime_info;
-  ThunarVfsFileSize  size_summary;
-  ThunarVfsFileSize  size;
-  GtkTreeIter        iter;
-  ThunarFile        *file;
-  GSList            *row;
-  GList             *lp;
-  gchar             *absolute_path;
-  gchar             *fspace_string;
-  gchar             *display_name;
-  gchar             *size_string;
-  gchar             *text;
-  gchar             *s;
-  gint               height;
-  gint               width;
-  gint               n;
+  const gchar *content_type;
+  const gchar *original_path;
+  GtkTreeIter  iter;
+  ThunarFile  *file;
+  guint64      size;
+  guint64      size_summary;
+  GSList      *row;
+  GList       *lp;
+  gchar       *absolute_path;
+  gchar       *fspace_string;
+  gchar       *display_name;
+  gchar       *size_string;
+  gchar       *text;
+  gchar       *s;
+  gint         height;
+  gint         width;
+  gint         n;
 
   _thunar_return_val_if_fail (THUNAR_IS_LIST_MODEL (store), NULL);
 
@@ -2228,7 +2292,11 @@ thunar_list_model_get_statusbar_text (ThunarListModel *store,
       if (G_LIKELY (file != NULL && thunar_file_get_free_space (file, &size)))
         {
           /* humanize the free space */
-          fspace_string = thunar_vfs_humanize_size (size, NULL, 0);
+#if GLIB_CHECK_VERSION (2, 30, 0)
+          fspace_string = g_format_size (size);
+#else
+          fspace_string = g_format_size_for_display (size);
+#endif
 
           /* check if we have atleast one file in this folder */
           if (G_LIKELY (store->nrows > 0))
@@ -2238,7 +2306,11 @@ thunar_list_model_get_statusbar_text (ThunarListModel *store,
                 size_summary += thunar_file_get_size (row->data);
 
               /* humanize the size summary */
-              size_string = thunar_vfs_humanize_size (size_summary, NULL, 0);
+#if GLIB_CHECK_VERSION (2, 30, 0)
+              size_string = g_format_size (size_summary);
+#else
+              size_string = g_format_size_for_display (size_summary);
+#endif
 
               /* generate a text which includes the size of all items in the folder */
               text = g_strdup_printf (ngettext ("%d item (%s), Free space: %s", "%d items (%s), Free space: %s", store->nrows),
@@ -2268,32 +2340,47 @@ thunar_list_model_get_statusbar_text (ThunarListModel *store,
 
       /* get the file for the given iter */
       file = THUNAR_FILE (G_SLIST (iter.user_data)->data);
+      
+      /* determine the content type of the file */
+      content_type = thunar_file_get_content_type (file);
 
       /* calculate the text to be displayed */
-      mime_info = thunar_file_get_mime_info (file);
       size_string = thunar_file_get_size_string (file);
-      if (G_UNLIKELY (strcmp (thunar_vfs_mime_info_get_name (mime_info), "inode/symlink") == 0))
+
+      if (G_UNLIKELY (content_type != NULL && g_str_equal (content_type, "inode/symlink")))
         {
           text = g_strdup_printf (_("\"%s\" broken link"), thunar_file_get_display_name (file));
         }
       else if (G_UNLIKELY (thunar_file_is_symlink (file)))
         {
           text = g_strdup_printf (_("\"%s\" (%s) link to %s"), thunar_file_get_display_name (file),
-                                  size_string, thunar_vfs_mime_info_get_comment (mime_info));
+                                  size_string, thunar_file_get_symlink_target (file));
+        }
+      else if (G_UNLIKELY (thunar_file_get_kind (file) == G_FILE_TYPE_SHORTCUT))
+        {
+          text = g_strdup_printf (_("\"%s\" shortcut"), thunar_file_get_display_name (file));
+        }
+      else if (G_UNLIKELY (thunar_file_get_kind (file) == G_FILE_TYPE_MOUNTABLE))
+        {
+          text = g_strdup_printf (_("\"%s\" mountable"), thunar_file_get_display_name (file));
         }
       else
         {
+          gchar *description;
+          
+          description = g_content_type_get_description (thunar_file_get_content_type (file));
           text = g_strdup_printf (_("\"%s\" (%s) %s"), thunar_file_get_display_name (file),
-                                  size_string, thunar_vfs_mime_info_get_comment (mime_info));
+                                  size_string, description);
+          g_free (description);
         }
       g_free (size_string);
 
       /* append the original path (if any) */
-      absolute_path = thunar_file_get_original_path (file);
-      if (G_UNLIKELY (absolute_path != NULL))
+      original_path = thunar_file_get_original_path (file);
+      if (G_UNLIKELY (original_path != NULL))
         {
           /* append the original path to the statusbar text */
-          display_name = g_filename_display_name (absolute_path);
+          display_name = g_filename_display_name (original_path);
           s = g_strdup_printf ("%s, %s %s", text, _("Original Path:"), display_name);
           g_free (display_name);
           g_free (text);
@@ -2302,16 +2389,17 @@ thunar_list_model_get_statusbar_text (ThunarListModel *store,
       else if (thunar_file_is_local (file) && thunar_file_is_regular (file))
         {
           /* check if we can determine the dimension of this file (only for image files) */
-          absolute_path = thunar_vfs_path_dup_string (thunar_file_get_path (file));
-          if (gdk_pixbuf_get_file_info (absolute_path, &width, &height) != NULL)
+          absolute_path = g_file_get_path (thunar_file_get_file (file));
+          if (absolute_path != NULL
+              && gdk_pixbuf_get_file_info (absolute_path, &width, &height) != NULL)
             {
               /* append the image dimensions to the statusbar text */
               s = g_strdup_printf ("%s, %s %dx%d", text, _("Image Size:"), width, height);
               g_free (text);
               text = s;
             }
+          g_free (absolute_path);
         }
-      g_free (absolute_path);
     }
   else
     {
@@ -2324,7 +2412,11 @@ thunar_list_model_get_statusbar_text (ThunarListModel *store,
 
       if (size_summary > 0)
         {
-          size_string = thunar_vfs_humanize_size (size_summary, NULL, 0);
+#if GLIB_CHECK_VERSION (2, 30, 0)
+          size_string = g_format_size (size_summary);
+#else
+          size_string = g_format_size_for_display (size_summary);
+#endif
           text = g_strdup_printf (ngettext ("%d item selected (%s)", "%d items selected (%s)", n), n, size_string);
           g_free (size_string);
         }
