@@ -1,20 +1,22 @@
-/* $Id$ */
+/* vi:set et ai sw=2 sts=2 ts=2: */
 /*-
  * Copyright (c) 2005-2006 Benedikt Meurer <benny@xfce.org>
+ * Copyright (c) 2009-2010 Jannis Pohlmann <jannis@xfce.org>
  *
- * This program is free software; you can redistribute it and/or modify it
- * under the terms of the GNU General Public License as published by the Free
- * Software Foundation; either version 2 of the License, or (at your option)
- * any later version.
+ * This program is free software; you can redistribute it and/or 
+ * modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation; either version 2 of 
+ * the License, or (at your option) any later version.
  *
- * This program is distributed in the hope that it will be useful, but WITHOUT
- * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
- * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
- * more details.
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
+ * GNU General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License along with
- * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
- * Place, Suite 330, Boston, MA  02111-1307  USA
+ * You should have received a copy of the GNU General Public 
+ * License along with this program; if not, write to the Free 
+ * Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -23,6 +25,7 @@
 
 #include <gdk/gdkkeysyms.h>
 
+#include <thunar/thunar-browser.h>
 #include <thunar/thunar-dialogs.h>
 #include <thunar/thunar-gobject-extensions.h>
 #include <thunar/thunar-gtk-extensions.h>
@@ -53,11 +56,9 @@ enum
 
 
 
-static void        thunar_location_entry_class_init            (ThunarLocationEntryClass *klass);
 static void        thunar_location_entry_component_init        (ThunarComponentIface     *iface);
 static void        thunar_location_entry_navigator_init        (ThunarNavigatorIface     *iface);
 static void        thunar_location_entry_location_bar_init     (ThunarLocationBarIface   *iface);
-static void        thunar_location_entry_init                  (ThunarLocationEntry      *location_entry);
 static void        thunar_location_entry_finalize              (GObject                  *object);
 static void        thunar_location_entry_get_property          (GObject                  *object,
                                                                 guint                     prop_id,
@@ -100,60 +101,11 @@ struct _ThunarLocationEntry
 
 
 
-static GObjectClass *thunar_location_entry_parent_class;
-
-
-
-GType
-thunar_location_entry_get_type (void)
-{
-  static GType type = G_TYPE_INVALID;
-
-  if (G_UNLIKELY (type == G_TYPE_INVALID))
-    {
-      static const GTypeInfo info =
-      {
-        sizeof (ThunarLocationEntryClass),
-        NULL,
-        NULL,
-        (GClassInitFunc) thunar_location_entry_class_init,
-        NULL,
-        NULL,
-        sizeof (ThunarLocationEntry),
-        0,
-        (GInstanceInitFunc) thunar_location_entry_init,
-        NULL,
-      };
-
-      static const GInterfaceInfo component_info =
-      {
-        (GInterfaceInitFunc) thunar_location_entry_component_init,
-        NULL,
-        NULL,
-      };
-
-      static const GInterfaceInfo navigator_info =
-      {
-        (GInterfaceInitFunc) thunar_location_entry_navigator_init,
-        NULL,
-        NULL,
-      };
-
-      static const GInterfaceInfo location_bar_info =
-      {
-        (GInterfaceInitFunc) thunar_location_entry_location_bar_init,
-        NULL,
-        NULL,
-      };
-
-      type = g_type_register_static (GTK_TYPE_HBOX, I_("ThunarLocationEntry"), &info, 0);
-      g_type_add_interface_static (type, THUNAR_TYPE_NAVIGATOR, &navigator_info);
-      g_type_add_interface_static (type, THUNAR_TYPE_COMPONENT, &component_info);
-      g_type_add_interface_static (type, THUNAR_TYPE_LOCATION_BAR, &location_bar_info);
-    }
-
-  return type;
-}
+G_DEFINE_TYPE_WITH_CODE (ThunarLocationEntry, thunar_location_entry, GTK_TYPE_HBOX,
+  G_IMPLEMENT_INTERFACE (THUNAR_TYPE_BROWSER, NULL)
+  G_IMPLEMENT_INTERFACE (THUNAR_TYPE_NAVIGATOR, thunar_location_entry_navigator_init)
+  G_IMPLEMENT_INTERFACE (THUNAR_TYPE_COMPONENT, thunar_location_entry_component_init)
+  G_IMPLEMENT_INTERFACE (THUNAR_TYPE_LOCATION_BAR, thunar_location_entry_location_bar_init))
 
 
 
@@ -163,9 +115,6 @@ thunar_location_entry_class_init (ThunarLocationEntryClass *klass)
   GtkBindingSet *binding_set;
   GObjectClass  *gobject_class;
 
-  /* determine the parent type class */
-  thunar_location_entry_parent_class = g_type_class_peek_parent (klass);
-
   gobject_class = G_OBJECT_CLASS (klass);
   gobject_class->finalize = thunar_location_entry_finalize;
   gobject_class->get_property = thunar_location_entry_get_property;
@@ -316,10 +265,14 @@ thunar_location_entry_set_property (GObject      *object,
                                     const GValue *value,
                                     GParamSpec   *pspec)
 {
+  ThunarLocationEntry *entry = THUNAR_LOCATION_ENTRY (object);
+
   switch (prop_id)
     {
     case PROP_CURRENT_DIRECTORY:
       thunar_navigator_set_current_directory (THUNAR_NAVIGATOR (object), g_value_get_object (value));
+      thunar_path_entry_set_working_directory (THUNAR_PATH_ENTRY (entry->path_entry), 
+                                               entry->current_directory);
       break;
 
     case PROP_SELECTED_FILES:
@@ -399,18 +352,16 @@ thunar_location_entry_accept_focus (ThunarLocationBar *location_bar,
 
 
 static void
-thunar_location_entry_activate (GtkWidget           *path_entry,
-                                ThunarLocationEntry *location_entry)
+thunar_location_entry_open_or_launch (ThunarLocationEntry *location_entry,
+                                      ThunarFile          *file)
 {
-  ThunarFile *file;
-  GError     *error = NULL;
+  GError *error = NULL;
 
   _thunar_return_if_fail (THUNAR_IS_LOCATION_ENTRY (location_entry));
-  _thunar_return_if_fail (location_entry->path_entry == path_entry);
+  _thunar_return_if_fail (THUNAR_IS_FILE (file));
 
-  /* determine the current file from the path entry */
-  file = thunar_path_entry_get_current_file (THUNAR_PATH_ENTRY (path_entry));
-  if (G_LIKELY (file != NULL))
+  /* check if the file is mounted */
+  if (thunar_file_is_mounted (file))
     {
       /* check if we have a new directory or a file to launch */
       if (thunar_file_is_directory (file))
@@ -421,17 +372,78 @@ thunar_location_entry_activate (GtkWidget           *path_entry,
       else
         {
           /* try to launch the selected file */
-          if (!thunar_file_launch (file, path_entry, &error))
-            {
-              thunar_dialogs_show_error (path_entry, error, _("Failed to launch \"%s\""), thunar_file_get_display_name (file));
-              g_error_free (error);
-            }
+          thunar_file_launch (file, location_entry->path_entry, NULL, &error);
 
           /* be sure to reset the current file of the path entry */
           if (G_LIKELY (location_entry->current_directory != NULL))
-            thunar_path_entry_set_current_file (THUNAR_PATH_ENTRY (path_entry), location_entry->current_directory);
+            {
+              thunar_path_entry_set_current_file (THUNAR_PATH_ENTRY (location_entry->path_entry), 
+                                                  location_entry->current_directory);
+            }
         }
     }
+  else
+    {
+      g_set_error (&error, G_IO_ERROR, G_IO_ERROR_NOT_FOUND, _("File does not exist"));
+    }
+
+  /* check if we need to display an error dialog */
+  if (error != NULL)
+    {
+      thunar_dialogs_show_error (location_entry->path_entry, error, 
+                                 _("Failed to open \"%s\""), 
+                                 thunar_file_get_display_name (file));
+      g_error_free (error);
+    }
+}
+
+
+
+static void
+thunar_location_entry_poke_file_finish (ThunarBrowser *browser,
+                                        ThunarFile    *file,
+                                        ThunarFile    *target_file,
+                                        GError        *error,
+                                        gpointer       ignored)
+{
+  _thunar_return_if_fail (THUNAR_IS_LOCATION_ENTRY (browser));
+  _thunar_return_if_fail (THUNAR_IS_FILE (file));
+
+  if (error == NULL)
+    {
+      /* try to open or launch the target file */
+      thunar_location_entry_open_or_launch (THUNAR_LOCATION_ENTRY (browser), 
+                                            target_file);
+    }
+  else
+    {
+      /* display an error explaining why we couldn't open/mount the file */
+      thunar_dialogs_show_error (THUNAR_LOCATION_ENTRY (browser)->path_entry,
+                                 error, _("Failed to open \"%s\""), 
+                                 thunar_file_get_display_name (file));
+    }
+}
+
+
+
+
+
+static void
+thunar_location_entry_activate (GtkWidget           *path_entry,
+                                ThunarLocationEntry *location_entry)
+{
+  ThunarFile *file;
+
+  _thunar_return_if_fail (THUNAR_IS_LOCATION_ENTRY (location_entry));
+  _thunar_return_if_fail (location_entry->path_entry == path_entry);
+
+  /* determine the current file from the path entry */
+  file = thunar_path_entry_get_current_file (THUNAR_PATH_ENTRY (path_entry));
+  if (G_LIKELY (file != NULL))
+    {
+      thunar_browser_poke_file (THUNAR_BROWSER (location_entry), file, path_entry,
+                                thunar_location_entry_poke_file_finish, NULL);
+    }
 }
 
 
@@ -497,15 +509,16 @@ thunar_location_entry_button_clicked (GtkWidget           *button,
 {
   ThunarShortcutsModel *model;
   ThunarIconFactory    *icon_factory;
-  ThunarVfsVolume      *volume;
   GtkIconTheme         *icon_theme;
-  const gchar          *icon_name;
   GtkTreeIter           iter;
   ThunarFile           *file;
   GtkWidget            *image;
   GtkWidget            *item;
   GtkWidget            *menu;
   GdkPixbuf            *icon;
+  GVolume              *volume;
+  GIcon                *volume_icon;
+  gchar                *volume_name;
   gint                  icon_size;
   gint                  width;
 
@@ -543,19 +556,16 @@ thunar_location_entry_button_clicked (GtkWidget           *button,
           else if (G_UNLIKELY (volume != NULL))
             {
               /* generate an image menu item for the volume */
-              item = gtk_image_menu_item_new_with_label (thunar_vfs_volume_get_name (volume));
+              volume_name = g_volume_get_name (volume);
+              item = gtk_image_menu_item_new_with_label (volume_name);
+              g_free (volume_name);
 
-              /* load the icon for the volume */
-              icon_name = thunar_vfs_volume_lookup_icon_name (volume, icon_theme);
-              icon = thunar_icon_factory_load_icon (icon_factory, icon_name, icon_size, NULL, FALSE);
-              if (G_LIKELY (icon != NULL))
-                {
-                  /* generate an image for the menu item */
-                  image = gtk_image_new_from_pixbuf (icon);
-                  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (item), image);
-                  g_object_unref (G_OBJECT (icon));
-                  gtk_widget_show (image);
-                }
+              /* generate an image for the menu item */
+              volume_icon = g_volume_get_icon (volume);
+              image = gtk_image_new_from_gicon (volume_icon, GTK_ICON_SIZE_MENU);
+              g_object_unref (volume_icon);
+              gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (item), image);
+              gtk_widget_show (image);
             }
           else
             {
@@ -566,16 +576,16 @@ thunar_location_entry_button_clicked (GtkWidget           *button,
               icon = thunar_icon_factory_load_file_icon (icon_factory, file, THUNAR_FILE_ICON_STATE_DEFAULT, icon_size);
               image = gtk_image_new_from_pixbuf (icon);
               gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (item), image);
-              g_object_unref (G_OBJECT (icon));
+              g_object_unref (icon);
               gtk_widget_show (image);
             }
 
           /* connect the file and volume to the item */
-          g_object_set_data_full (G_OBJECT (item), I_("thunar-vfs-volume"), volume, g_object_unref);
+          g_object_set_data_full (G_OBJECT (item), I_("volume"), volume, g_object_unref);
           g_object_set_data_full (G_OBJECT (item), I_("thunar-file"), file, g_object_unref);
 
           /* append the new item to the menu */
-          g_signal_connect (G_OBJECT (item), "activate", G_CALLBACK (thunar_location_entry_item_activated), location_entry);
+          g_signal_connect (item, "activate", G_CALLBACK (thunar_location_entry_item_activated), location_entry);
           gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
           gtk_widget_show (item);
         }
@@ -610,14 +620,77 @@ thunar_location_entry_button_clicked (GtkWidget           *button,
 
 
 
+static void
+thunar_location_entry_mount_finish (GObject      *object,
+                                    GAsyncResult *result,
+                                    gpointer      user_data)
+{
+  ThunarLocationEntry *location_entry = THUNAR_LOCATION_ENTRY (user_data);
+  ThunarFile          *file = NULL;
+  GtkWidget           *window;
+  GVolume             *volume = G_VOLUME (object);
+  GError              *error = NULL;
+  GMount              *mount;
+  GFile               *mount_point;
+  gchar               *volume_name;
+
+  _thunar_return_if_fail (G_IS_VOLUME (object));
+  _thunar_return_if_fail (THUNAR_IS_LOCATION_ENTRY (user_data));
+
+  if (!g_volume_mount_finish (volume, result, &error))
+    {
+      /* determine the toplevel window */
+      window = gtk_widget_get_toplevel (GTK_WIDGET (location_entry));
+
+      volume_name = g_volume_get_name (volume);
+      thunar_dialogs_show_error (window, error, _("Failed to mount \"%s\""), volume_name);
+      g_free (volume_name);
+    }
+  else
+    {
+      mount = g_volume_get_mount (volume);
+      if (mount != NULL)
+        {
+          mount_point = g_mount_get_root (mount);
+          file = thunar_file_get (mount_point, NULL);
+          g_object_unref (mount_point);
+
+          /* check if we have a file object now */
+          if (G_LIKELY (file != NULL))
+            {
+              /* make sure that this is actually a directory */
+              if (thunar_file_is_directory (file))
+                {
+                  /* open the new directory */
+                  thunar_navigator_change_directory (THUNAR_NAVIGATOR (location_entry), 
+                                                     file);
+                }
+
+              /* cleanup */
+              g_object_unref (file);
+            }
+
+          g_object_unref (mount);
+        }
+    }
+
+  g_object_unref (location_entry);
+}
+
+
+
 static void
 thunar_location_entry_item_activated (GtkWidget           *item,
                                       ThunarLocationEntry *location_entry)
 {
-  ThunarVfsVolume *volume;
-  ThunarFile      *file;
+  GMountOperation *mount_operation;
+  ThunarFile      *file = NULL;
   GtkWidget       *window;
+  GVolume         *volume;
   GError          *error = NULL;
+  GMount          *mount;
+  GFile           *mount_point;
+  gchar           *volume_name;
 
   _thunar_return_if_fail (GTK_IS_MENU_ITEM (item));
   _thunar_return_if_fail (THUNAR_IS_LOCATION_ENTRY (location_entry));
@@ -626,30 +699,43 @@ thunar_location_entry_item_activated (GtkWidget           *item,
   window = gtk_widget_get_toplevel (GTK_WIDGET (location_entry));
 
   /* check if the item corresponds to a volume */
-  volume = g_object_get_data (G_OBJECT (item), "thunar-vfs-volume");
+  volume = g_object_get_data (G_OBJECT (item), "volume");
   if (G_UNLIKELY (volume != NULL))
     {
       /* check if the volume isn't already mounted */
-      if (G_LIKELY (!thunar_vfs_volume_is_mounted (volume)))
+      if (G_LIKELY (!thunar_g_volume_is_mounted (volume)))
         {
-          /* try to mount the volume */
-          if (!thunar_vfs_volume_mount (volume, window, &error))
-            {
-              /* display an error dialog to inform the user */
-              thunar_dialogs_show_error (window, error, _("Failed to mount \"%s\""), thunar_vfs_volume_get_name (volume));
-              g_error_free (error);
-              return;
-            }
-        }
+          mount_operation = gtk_mount_operation_new (GTK_WINDOW (window));
 
-      /* try to determine the mount point of the volume */
-      file = thunar_file_get_for_path (thunar_vfs_volume_get_mount_point (volume), &error);
-      if (G_UNLIKELY (file == NULL))
+          g_volume_mount (volume, G_MOUNT_MOUNT_NONE, mount_operation, NULL,
+                          thunar_location_entry_mount_finish, 
+                          g_object_ref (location_entry));
+
+          g_object_unref (mount_operation);
+        }
+      else
         {
-          /* display an error dialog to inform the user */
-          thunar_dialogs_show_error (window, error, _("Failed to determine the mount point for %s"), thunar_vfs_volume_get_name (volume));
-          g_error_free (error);
-          return;
+          mount = g_volume_get_mount (volume);
+          if (mount != NULL)
+            {
+              /* try to determine the mount point of the volume */
+              mount_point = g_mount_get_root (mount);
+              file = thunar_file_get (mount_point, &error);
+              g_object_unref (mount_point);
+              
+              if (G_UNLIKELY (file == NULL))
+                {
+                  /* display an error dialog to inform the user */
+                  volume_name = g_volume_get_name (volume);
+                  thunar_dialogs_show_error (window, error, 
+                                             _("Failed to determine the mount point of \"%s\""), 
+                                             volume_name);
+                  g_free (volume_name);
+                  g_error_free (error);
+                }
+
+              g_object_unref (mount);
+            }
         }
     }
   else
