@@ -1,6 +1,7 @@
 /* $Id$ */
 /*-
  * Copyright (c) 2005-2006 Benedikt Meurer <benny@xfce.org>
+ * Copyright (c) 2009 Jannis Pohlmann <jannis@xfce.org>
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the Free
@@ -62,9 +63,7 @@ typedef struct _ThunarRenamerModelItem ThunarRenamerModelItem;
 
 
 
-static void                    thunar_renamer_model_class_init          (ThunarRenamerModelClass *klass);
 static void                    thunar_renamer_model_tree_model_init     (GtkTreeModelIface       *iface);
-static void                    thunar_renamer_model_init                (ThunarRenamerModel      *renamer_model);
 static void                    thunar_renamer_model_finalize            (GObject                 *object);
 static void                    thunar_renamer_model_get_property        (GObject                 *object,
                                                                          guint                    prop_id,
@@ -116,7 +115,7 @@ static gboolean                thunar_renamer_model_conflict_item       (ThunarR
                                                                          ThunarRenamerModelItem  *item);
 static gchar                  *thunar_renamer_model_process_item        (ThunarRenamerModel      *renamer_model,
                                                                          ThunarRenamerModelItem  *item,
-                                                                         guint                    index);
+                                                                         guint                    idx);
 static gboolean                thunar_renamer_model_update_idle         (gpointer                 user_data);
 static void                    thunar_renamer_model_update_idle_destroy (gpointer                 user_data);
 static ThunarRenamerModelItem *thunar_renamer_model_item_new            (ThunarFile              *file) G_GNUC_MALLOC;
@@ -158,54 +157,18 @@ struct _ThunarRenamerModel
 
 struct _ThunarRenamerModelItem
 {
-  ThunarVfsInfo *info;
-  ThunarFile    *file;
-  gchar         *name;
-  guint          changed : 1;  /* if the file changed */
-  guint          conflict : 1; /* if the item conflicts with another item */
-  guint          dirty : 1;    /* if the item must be updated */
+  ThunarFile *file;
+  gchar      *name;
+  guint64     date_changed;
+  guint       changed : 1;  /* if the file changed */
+  guint       conflict : 1; /* if the item conflicts with another item */
+  guint       dirty : 1;    /* if the item must be updated */
 };
 
 
 
-static GObjectClass *thunar_renamer_model_parent_class;
-
-
-
-GType
-thunar_renamer_model_get_type (void)
-{
-  static GType type = G_TYPE_INVALID;
-
-  if (G_UNLIKELY (type == G_TYPE_INVALID))
-    {
-      static const GTypeInfo info =
-      {
-        sizeof (ThunarRenamerModelClass),
-        NULL,
-        NULL,
-        (GClassInitFunc) thunar_renamer_model_class_init,
-        NULL,
-        NULL,
-        sizeof (ThunarRenamerModel),
-        0,
-        (GInstanceInitFunc) thunar_renamer_model_init,
-        NULL,
-      };
-
-      static const GInterfaceInfo tree_model_info =
-      {
-        (GInterfaceInitFunc) thunar_renamer_model_tree_model_init,
-        NULL,
-        NULL,
-      };
-
-      type = g_type_register_static (G_TYPE_OBJECT, I_("ThunarRenamerModel"), &info, 0);
-      g_type_add_interface_static (type, GTK_TYPE_TREE_MODEL, &tree_model_info);
-    }
-
-  return type;
-}
+G_DEFINE_TYPE_WITH_CODE (ThunarRenamerModel, thunar_renamer_model, G_TYPE_OBJECT,
+    G_IMPLEMENT_INTERFACE (GTK_TYPE_TREE_MODEL, thunar_renamer_model_tree_model_init))
 
 
 
@@ -214,9 +177,6 @@ thunar_renamer_model_class_init (ThunarRenamerModelClass *klass)
 {
   GObjectClass *gobject_class;
 
-  /* determine the parent type class */
-  thunar_renamer_model_parent_class = g_type_class_peek_parent (klass);
-
   gobject_class = G_OBJECT_CLASS (klass);
   gobject_class->finalize = thunar_renamer_model_finalize;
   gobject_class->get_property = thunar_renamer_model_get_property;
@@ -479,17 +439,17 @@ thunar_renamer_model_get_path (GtkTreeModel *tree_model,
                                GtkTreeIter  *iter)
 {
   ThunarRenamerModel *renamer_model = THUNAR_RENAMER_MODEL (tree_model);
-  gint                index;
+  gint                idx;
 
   _thunar_return_val_if_fail (THUNAR_IS_RENAMER_MODEL (renamer_model), NULL);
   _thunar_return_val_if_fail (iter->stamp == renamer_model->stamp, NULL);
 
-  /* determine the index of the item */
-  index = g_list_position (renamer_model->items, iter->user_data);
-  if (G_UNLIKELY (index < 0))
+  /* determine the idx of the item */
+  idx = g_list_position (renamer_model->items, iter->user_data);
+  if (G_UNLIKELY (idx < 0))
     return NULL;
 
-  return gtk_tree_path_new_from_indices (index, -1);
+  return gtk_tree_path_new_from_indices (idx, -1);
 }
 
 
@@ -633,6 +593,7 @@ thunar_renamer_model_file_changed (ThunarRenamerModel *renamer_model,
   GtkTreePath            *path;
   GtkTreeIter             iter;
   GList                  *lp;
+  guint64                 date_changed;
 
   _thunar_return_if_fail (THUNAR_IS_FILE (file));
   _thunar_return_if_fail (THUNAR_IS_FILE_MONITOR (file_monitor));
@@ -643,24 +604,27 @@ thunar_renamer_model_file_changed (ThunarRenamerModel *renamer_model,
   for (lp = renamer_model->items; lp != NULL; lp = lp->next)
     if (THUNAR_RENAMER_MODEL_ITEM (lp->data)->file == file)
       {
-        /* check if the file changed on disk */
         item = THUNAR_RENAMER_MODEL_ITEM (lp->data);
-        if (!thunar_vfs_info_matches (item->info, thunar_file_get_info (file)))
+
+        /* check if the file changed on disk, this is done to prevent
+         * excessive looping when some renamers are used
+         * (thunar-media-tags-plugin is an example) */
+        date_changed = thunar_file_get_date (file, THUNAR_FILE_DATE_CHANGED);
+        if (item->date_changed == date_changed)
+          break;
+
+        /* check if we're frozen */
+        if (G_LIKELY (!renamer_model->frozen))
           {
-            /* connect to the new info */
-            thunar_vfs_info_unref (item->info);
-            item->info = thunar_vfs_info_ref (thunar_file_get_info (file));
+            /* the file changed */
+            item->changed = TRUE;
 
-            /* check if we're frozen */
-            if (G_LIKELY (!renamer_model->frozen))
-              {
-                /* the file changed */
-                THUNAR_RENAMER_MODEL_ITEM (lp->data)->changed = TRUE;
+            /* set the new mtime */
+            item->date_changed = date_changed;
 
-                /* invalidate the item */
-                thunar_renamer_model_invalidate_item (renamer_model, lp->data);
-                break;
-              }
+            /* invalidate the item */
+            thunar_renamer_model_invalidate_item (renamer_model, item);
+            break;
           }
 
         /* determine the iter for the item */
@@ -683,7 +647,7 @@ thunar_renamer_model_file_destroyed (ThunarRenamerModel *renamer_model,
 {
   GtkTreePath *path;
   GList       *lp;
-  gint         index;
+  gint         idx;
 
   _thunar_return_if_fail (THUNAR_IS_RENAMER_MODEL (renamer_model));
   _thunar_return_if_fail (renamer_model->file_monitor == file_monitor);
@@ -694,8 +658,8 @@ thunar_renamer_model_file_destroyed (ThunarRenamerModel *renamer_model,
   for (lp = renamer_model->items; lp != NULL; lp = lp->next)
     if (THUNAR_RENAMER_MODEL_ITEM (lp->data)->file == file)
       {
-        /* determine the index of the item */
-        index = g_list_position (renamer_model->items, lp);
+        /* determine the idx of the item */
+        idx = g_list_position (renamer_model->items, lp);
 
         /* free the item data */
         thunar_renamer_model_item_free (lp->data);
@@ -704,7 +668,7 @@ thunar_renamer_model_file_destroyed (ThunarRenamerModel *renamer_model,
         renamer_model->items = g_list_delete_link (renamer_model->items, lp);
 
         /* tell the view that the item is gone */
-        path = gtk_tree_path_new_from_indices (index, -1);
+        path = gtk_tree_path_new_from_indices (idx, -1);
         gtk_tree_model_row_deleted (GTK_TREE_MODEL (renamer_model), path);
         gtk_tree_path_free (path);
 
@@ -753,15 +717,15 @@ static gboolean
 trm_same_directory (ThunarFile *a,
                     ThunarFile *b)
 {
-  ThunarVfsPath *parent_a;
-  ThunarVfsPath *parent_b;
+  GFile *parent_a;
+  GFile *parent_b;
 
   /* determine the parent paths for both files */
-  parent_a = thunar_vfs_path_get_parent (thunar_file_get_path (a));
-  parent_b = thunar_vfs_path_get_parent (thunar_file_get_path (b));
+  parent_a = g_file_get_parent (thunar_file_get_file (a));
+  parent_b = g_file_get_parent (thunar_file_get_file (b));
 
   /* check if both files have the same parent */
-  return (parent_a != NULL && parent_b != NULL && thunar_vfs_path_equal (parent_a, parent_b));
+  return (parent_a != NULL && parent_b != NULL && g_file_equal (parent_a, parent_b));
 }
 
 
@@ -817,7 +781,7 @@ thunar_renamer_model_conflict_item (ThunarRenamerModel     *renamer_model,
 static gchar*
 thunar_renamer_model_process_item (ThunarRenamerModel     *renamer_model,
                                    ThunarRenamerModelItem *item,
-                                   guint                   index)
+                                   guint                   idx)
 {
   ThunarRenamerMode mode;
   const gchar      *display_name;
@@ -856,7 +820,7 @@ thunar_renamer_model_process_item (ThunarRenamerModel     *renamer_model,
           text = g_strndup (display_name, (dot - display_name));
 
           /* determine the new name */
-          prefix = thunarx_renamer_process (renamer_model->renamer, THUNARX_FILE_INFO (item->file), text, index);
+          prefix = thunarx_renamer_process (renamer_model->renamer, THUNARX_FILE_INFO (item->file), text, idx);
 
           /* determine the new full name */
           name = g_strconcat (prefix, dot, NULL);
@@ -868,12 +832,11 @@ thunar_renamer_model_process_item (ThunarRenamerModel     *renamer_model,
 
         case THUNAR_RENAMER_MODE_SUFFIX:
           /* determine the new suffix */
-          suffix = thunarx_renamer_process (renamer_model->renamer, THUNARX_FILE_INFO (item->file), dot + 1, index);
+          suffix = thunarx_renamer_process (renamer_model->renamer, THUNARX_FILE_INFO (item->file), dot + 1, idx);
 
-          /* generate the new file name */
-          name = g_new (gchar, (dot - display_name) + 1 + strlen (suffix) + 1);
-          memcpy (name, display_name, (dot - display_name) + 1);
-          memcpy (name + (dot - display_name) + 1, suffix, strlen (suffix) + 1);
+          prefix = g_strndup (display_name, (dot - display_name) + 1);
+          name = g_strconcat (prefix, suffix, NULL);
+          g_free (prefix);
 
           /* release the suffix */
           g_free (suffix);
@@ -881,7 +844,7 @@ thunar_renamer_model_process_item (ThunarRenamerModel     *renamer_model,
 
         case THUNAR_RENAMER_MODE_BOTH:
           /* determine the new full name */
-          name = thunarx_renamer_process (renamer_model->renamer, THUNARX_FILE_INFO (item->file), display_name, index);
+          name = thunarx_renamer_process (renamer_model->renamer, THUNARX_FILE_INFO (item->file), display_name, idx);
           break;
 
         default:
@@ -912,7 +875,7 @@ thunar_renamer_model_update_idle (gpointer user_data)
   GtkTreeIter             iter;
   gboolean                changed = FALSE;
   gboolean                conflict;
-  guint                   index;
+  guint                   idx;
   gchar                  *name;
   GList                  *lp;
 
@@ -922,7 +885,7 @@ thunar_renamer_model_update_idle (gpointer user_data)
   if (G_LIKELY (!renamer_model->frozen))
     {
       /* process the first dirty item */
-      for (index = 0, lp = renamer_model->items; !changed && lp != NULL; ++index, lp = lp->next)
+      for (idx = 0, lp = renamer_model->items; !changed && lp != NULL; ++idx, lp = lp->next)
         {
           /* check if this item is dirty */
           item = THUNAR_RENAMER_MODEL_ITEM (lp->data);
@@ -937,7 +900,7 @@ thunar_renamer_model_update_idle (gpointer user_data)
           item->dirty = FALSE;
 
           /* determine the new name for the item */
-          name = thunar_renamer_model_process_item (renamer_model, item, index);
+          name = thunar_renamer_model_process_item (renamer_model, item, idx);
           if (!exo_str_is_equal (item->name, name))
             {
               /* apply new name */
@@ -971,7 +934,7 @@ thunar_renamer_model_update_idle (gpointer user_data)
               GTK_TREE_ITER_INIT (iter, renamer_model->stamp, lp);
 
               /* emit "row-changed" for this item */
-              path = gtk_tree_path_new_from_indices (index, -1);
+              path = gtk_tree_path_new_from_indices (idx, -1);
               gtk_tree_model_row_changed (GTK_TREE_MODEL (renamer_model), path, &iter);
               gtk_tree_path_free (path);
             }
@@ -1003,9 +966,9 @@ thunar_renamer_model_item_new (ThunarFile *file)
 {
   ThunarRenamerModelItem *item;
 
-  item = _thunar_slice_new0 (ThunarRenamerModelItem);
-  item->info = thunar_vfs_info_ref (thunar_file_get_info (file));
+  item = g_slice_new0 (ThunarRenamerModelItem);
   item->file = g_object_ref (G_OBJECT (file));
+  item->date_changed = thunar_file_get_date (file, THUNAR_FILE_DATE_CHANGED);
   item->dirty = TRUE;
 
   return item;
@@ -1017,9 +980,8 @@ static void
 thunar_renamer_model_item_free (ThunarRenamerModelItem *item)
 {
   g_object_unref (G_OBJECT (item->file));
-  thunar_vfs_info_unref (item->info);
   g_free (item->name);
-  _thunar_slice_free (ThunarRenamerModelItem, item);
+  g_slice_free (ThunarRenamerModelItem, item);
 }
 
 
